

//////////////////////////////////////////////////////////////////
//                                                              //
//           PLINK (c) 2005-2006 Shaun Purcell                  //
//                                                              //
// This file is distributed under the GNU General Public        //
// License, Version 2.  Please see the file COPYING for more    //
// details                                                      //
//                                                              //
//////////////////////////////////////////////////////////////////



#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <cmath>
#include <vector>
#include <map>

#include "plink.h"
#include "options.h"
#include "phase.h"
#include "helper.h"



//////////////////////////////
// Unweighted tests

void HaploPhase::haplotypicTDT(map<int,int> & tests, int nt, bool display)
{
  
  // No implementation of TDT omnibus test yet
  if (nt!=2) return;

  // This might be a haplotype-specific test (i.e. of a single haplotype)
  // or of a group of haplotypes versus the rest
  
  // Find test haplotype(s)
  
  int hh=0;
  double tr = 0;
  double un = 0;

  // If there is >1 haplotype, we will not be reporting in any case,
  // so does not matter that 'hh' will be wrong

  map<int,int>::iterator i1 = tests.begin();
  while ( i1 != tests.end() )
    {
      if ( i1->second == 0 )
	{
	  tr += trans[i1->first];
	  un += untrans[i1->first];
	  hh = i1->first;
	}

      i1++;
    }
  
  
  // 'result' visible outside this class, (b-c)^2/(b+c)

  result  = tr - un;
  result *= result;
  result /= tr + un;
  pvalue = chiprobP(result,1);

  if ( display)
    HTEST << setw(10) << hname << " "
	  << setw(12) << haplotypeName(hh) << " "
	  << setw(10) << trans[hh] << " "
	  << setw(10) << untrans[hh] << " ";


  if ( display )
    {
      if ( realnum(result) )
	{
	  HTEST << setw(10) << result << " "
		<< setw(10) << pvalue << " ";
	}
      else
	{
	  HTEST << setw(10) << "NA" << " "
		<< setw(10) << "NA" << " ";
	}
      
      for (int snps=0; snps<ns-1; snps++)
	HTEST << P.locus[S[snps]]->name << "|";
      
      HTEST << P.locus[S[ns-1]]->name << "\n";
    }
  
  return;
}



void HaploPhase::haplotypicWeightedTDT()
{

  vector_t weights;
  
  for (int i=0; i<nh; i++)
    {
      map<string,double>::iterator 
	whap = new_pred_weighted_allele[current].find( haplotypeName(i) );
      
      if ( whap != new_pred_weighted_allele[current].end() )
	{
	  weights.push_back( whap->second );
	}
      else
	{
	  weights.push_back( 0 );
	}
    }

  
  double T = 0;
  double U = 0;
  
  for (int h=0; h<nh; h++)
    {
      T += trans[h] * weights[h];
      U += untrans[h] * weights[h];
    }
  
  double chisq = ( (T-U)*(T-U) )  / ( T+U ) ;
  
  HTEST << setw(10) << hname << " "
	<< setw(12) << new_map[current]->allele1 << " "
	<< setw(10) << T << " " 
	<< setw(10) << U << " ";
  
  if ( realnum(chisq) )
    {
      HTEST << setw(10) << chisq << " "
	    << setw(10) << chiprobP(chisq,1) << " ";
    }
  else
    {
      HTEST << setw(10) << "NA" << " "
	    << setw(10) << "NA" << " ";
    }
  
  for (int snps=0; snps<ns-1; snps++)
    HTEST << P.locus[S[snps]]->name << "|";
  
  HTEST << P.locus[S[ns-1]]->name << "\n";
  
  return;

}

